unit JoystickUnit;

{
  RCX Joystick
  ============

  Written by Mark Overmars

  This is a very simple program to control the RCX using the
  joystick. Motions are sort of as expected. The buttons start
  the corresponding tasks in the RCX.

  This is a stand-alone version of the joystick in the RCX Command
  Center. It is meant for demonstration purposes and, hence, freeware.

  see http://www.cs.uu.nl/~markov/lego/ for more information.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, OleCtrls, SPIRITLib_TLB, ComCtrls;

type
  TJoystickForm = class(TForm)
    JoyTimer: TTimer;
    RCX: TSpirit;
    GroupBox1: TGroupBox;
    LMotorA: TRadioButton;
    LMotorB: TRadioButton;
    LMotorC: TRadioButton;
    LReversed: TCheckBox;
    GroupBox2: TGroupBox;
    RMotorA: TRadioButton;
    RMotorB: TRadioButton;
    RMotorC: TRadioButton;
    RReversed: TCheckBox;
    GroupBox3: TGroupBox;
    SpeedBar: TTrackBar;
    procedure JoyTimerTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure LReversedClick(Sender: TObject);
    procedure RReversedClick(Sender: TObject);
    procedure SpeedBarChange(Sender: TObject);
    procedure RMotorClick(Sender: TObject);
    procedure LMotorClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  JoystickForm: TJoystickForm;

implementation

{$R *.DFM}

uses MMSystem;

var LeftMotor:integer =0;                  // the left motor
    RightMotor:integer =2;                 // the right motor
    LeftReversed:boolean =false;           // whether left must be reversed
    RightReversed:boolean =false;          // whether right must be reversed

var oldldir:integer =100;                   // previous left motor direction
    oldrdir:integer =100;                   // previous right motor direction
    oldbuttons:array[1..4] of boolean=      // previous status of buttons
               (false,false,false,false) ;



procedure SetRCXSpeed(xx,yy:integer);
{Computes the correct directions and set the motors}
var ldir,rdir:integer;
    lstr,rstr:string;
begin
  {Convert joystick values to directions for the motors}
  if (yy<20000) then
  begin
    if (xx<20000) then      begin ldir:= 0; rdir:= 1; end
    else if (xx>44000) then begin ldir:= 1; rdir:= 0; end
    else                    begin ldir:= 1; rdir:= 1; end;
  end else if (yy>44000) then
  begin
    if (xx<20000) then      begin ldir:= 0; rdir:=-1; end
    else if (xx>44000) then begin ldir:=-1; rdir:= 0; end
    else                    begin ldir:=-1; rdir:=-1; end;
  end else begin
    if (xx<20000) then      begin ldir:=-1; rdir:= 1; end
    else if (xx>44000) then begin ldir:= 1; rdir:=-1; end
    else                    begin ldir:= 0; rdir:= 0; end;
  end;
  if LeftReversed then ldir := -ldir;
  if RightReversed then rdir := -rdir;

  {Execute the motions}
  lstr:=IntToStr(LeftMotor);
  rstr:=IntToStr(RightMotor);
  with JoystickForm do
  begin
    if ldir <> oldldir then
    begin
      if ldir=0 then RCX.Off(lstr);
      if ldir>0 then begin RCX.SetFwd(lstr); RCX.On_(lstr); end;
      if ldir<0 then begin RCX.SetRwd(lstr); RCX.On_(lstr); end;
    end;
    if rdir <> oldrdir then
    begin
      if rdir=0 then RCX.Off(rstr);
      if rdir>0 then begin RCX.SetFwd(rstr); RCX.On_(rstr); end;
      if rdir<0 then begin RCX.SetRwd(rstr); RCX.On_(rstr); end;
    end;
  end;

  {Save the values}
  oldldir := ldir;
  oldrdir := rdir;
end;

procedure TJoystickForm.JoyTimerTimer(Sender: TObject);
{ The timer callback that does the actual work}
var joyinfo:tjoyinfo;
    buttons: array [1..4] of boolean;
    i:integer;
begin
  {Get the joystick value and check for errors}
  if joygetpos(JOYSTICKID1,@joyinfo) <> JOYERR_NOERROR then
  begin
    ShowMessage('Cannot find the joystick.');
    JoyTimer.Enabled := false;
    Exit;
  end;

  {Handle the position}
  SetRCXSpeed(joyinfo.wXpos,joyinfo.wYpos);

  {Handle the buttons}
  buttons[1] := (JOY_BUTTON1 and joyinfo.wButtons) > 0;
  buttons[2] := (JOY_BUTTON2 and joyinfo.wButtons) > 0;
  buttons[3] := (JOY_BUTTON3 and joyinfo.wButtons) > 0;
  buttons[4] := (JOY_BUTTON4 and joyinfo.wButtons) > 0;
  for i:=1 to 4 do
    if buttons[i] <> oldbuttons[i] then
    begin
      oldbuttons[i] := buttons[i];
      if buttons[i] then JoystickForm.RCX.StartTask(i);
    end;
end;


procedure TJoystickForm.LMotorClick(Sender: TObject);
{Left motor selection}
begin
  LeftMotor := TRadioButton(Sender).Tag;
  RCX.SetPower(IntToStr(LeftMotor),2,SpeedBar.Position);
end;

procedure TJoystickForm.LReversedClick(Sender: TObject);
{Left Reverse selection}
begin
  LeftReversed := LReversed.Checked;
end;

procedure TJoystickForm.RMotorClick(Sender: TObject);
{Right motor selection}
begin
  RightMotor := TRadioButton(Sender).Tag;
  RCX.SetPower(IntToStr(RightMotor),2,SpeedBar.Position);
end;

procedure TJoystickForm.RReversedClick(Sender: TObject);
{Right reverse selection}
begin
  RightReversed := RReversed.Checked;
end;

procedure TJoystickForm.SpeedBarChange(Sender: TObject);
{Speed selection}
begin
  RCX.SetPower(IntToStr(LeftMotor)+IntToStr(RightMotor),2,SpeedBar.Position);
end;

procedure TJoystickForm.FormShow(Sender: TObject);
{Form becomes visible}
var i:integer;
begin
  {Find the right port}
  for i:=1 to 4 do
  begin
    RCX.ComPortNo := i;
    if RCX.InitComm() then break;
  end;
  RCX.SetPower(IntToStr(LeftMotor)+IntToStr(RightMotor),2,SpeedBar.Position);
  {Start the timer}
  JoyTimer.Enabled := true;
end;

procedure TJoystickForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
{Form stops from being visible}
begin
  {Stop the timer}
  JoyTimer.Enabled := false;
  {Close the connection to the RCX}
  RCX.CloseComm();
end;

end.
